package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.Text; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.util.Map;
//import java.util.Map;
import java.util.Iterator;

public class QueryAction extends Action {
    

    public String process (Element message) {


	// create the return page tree
	Element page = doc_.createElement("page");
	page.setAttribute("lang", message.getAttribute("lang"));
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, "translate"), true));
	// add the system stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, "config"), true));

	Element request = (Element)message.getElementsByTagName("request").item(0);
	String info = request.getAttribute("info");
	// remove the q/ bit
	info = GSPath.removeFirstLink(info);
	String query_type = GSPath.getFirstLink(info);
	
	if (query_type.equals("")) {
	    query_type = "text";
	}

	if (query_type.equals("text")) {
	    return textQuery(page, request);
	}
	if (query_type.equals("field")) {
	    return fieldQuery(page, request);
	}
	return unknownQuery(page, request);

	
    }
    
    // we may be able to generalize this for all query types?
    protected String textQuery(Element page, Element request) {

	// chack that the stylesheet is present - cant output a page without one. we may adapt this to use unknownquery stylesheet? - or ask for one from the MR
	String stylesheet = GSFile.stylesheetPath(config_, "textquery.xsl");
	if (stylesheet==null) {
	    return GSHTML.errorPage("textquery stylesheet not found!");
	}

	// extract the params from the cgi-request, and check that we have a coll specified
	// first convert short to long names
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, "paramList");
	cgi_.toLong(cgi_paramList);
	HashMap params = GSXML.extractParams(cgi_paramList);

	String collection = (String)params.get("c");
	if (collection == null || collection.equals("")) {
	    return GSHTML.errorPage("textquery, need to specify a colleciton!");
	}

	// the first part of the data for the page is the cgi-params
	Element cgi_request = (Element)doc_.importNode(request, true);
	page.appendChild(cgi_request);

	// get the service info from the MR - this will probably need to be cached somehow later on. and add as a description node to the cgi-request - this doesn't work. change to add the specified value as a current attribute to the param - need to do somthing different for query

	Element mr_info_message = doc_.createElement("message");
	Element mr_info_request = doc_.createElement("request");
	mr_info_message.appendChild(mr_info_request);
	mr_info_request.setAttribute("type", "describe");

	String to = collection;
	to = GSPath.appendLink(to, "TextQuery");
	mr_info_request.setAttribute("to", to);

	Element mr_info_response = (Element) mr_.process(mr_info_message);
	System.err.println("describe response for service");
	System.err.println(converter_.getString(mr_info_response));
	Element description = doc_.createElement("description");
	// just param list for now - may need content info too
	Node pl = GSXML.getNodeByPath(mr_info_response, "response/service/paramList");
	if (pl ==null) {
	    System.out.println("pl is null");
	}
	else {
	    System.out.println("pl name is "+pl.getNodeName());
	}
	Element imported_paramList = (Element)doc_.importNode(pl, true);
	// convert the name to short
	cgi_.addShortNames(imported_paramList);

	description.appendChild(imported_paramList);
	
	// for each param in page_request, set the current value if present
	//actually, prob dont need to know the default value, just which one to display - overwrite the default att?
	
	Element param = (Element)imported_paramList.getFirstChild();
	while (param !=null) {
	    if (param.getNodeName().equals("param")) { // just in case
		String name = param.getAttribute("name");
		String current = (String)params.get(name);
		if (current !=null && !current.equals("")) {
		    param.setAttribute("default", current);
		}
	    }
	    param = (Element)param.getNextSibling();
	}
	
	cgi_request.appendChild(description);
	
	// check the query string
	String query = (String)params.get("q");

	if (query==null|| query.equals("")) {
	    // if there is no q set, just output the search form
	    return transformer_.transform(stylesheet, page);       
	}
	
	// else do the request, output search box and results
	Element mr_query_message = doc_.createElement("message");
	Element mr_query_request = doc_.createElement("request");
	mr_query_message.appendChild(mr_query_request);

	mr_query_request.setAttribute("type", "query");
	mr_query_request.setAttribute("to", to);

	// paramList
	Element paramList = doc_.createElement("paramList");
	mr_query_request.appendChild(paramList);

	Iterator i = params.entrySet().iterator();
	while (i.hasNext()) {
	    Map.Entry e = (Map.Entry)i.next();
	    String name = cgi_.toLong((String)e.getKey());
	    if (!name.equals("")) {
		String value = (String)e.getValue();
	    // if (!name.equals("c")) {
		Element p = doc_.createElement("param");
		p.setAttribute("name", name);
		p.setAttribute("value", value);
		paramList.appendChild(p);
		// }
	    }
	}

	// content - contains the query string
	Element query_content = doc_.createElement("content");
	Text query_text = doc_.createTextNode(query);
	query_content.appendChild(query_text);
	mr_query_request.appendChild(query_content);
	
	Element mr_query_response = (Element)mr_.process(mr_query_message);

	// this result is the list of docs.
	// now take that list, and get the Titles
	// for now, just create a whole new request

	// check that there are some resources - for now check the list, but later should use a numdocs metadata elem
	Element resource_list = (Element)GSXML.getNodeByPath(mr_query_response, "response/content/resourceList"); // resourceList not present if no docs found
	if (resource_list == null) {

	    Element result_response = (Element)GSXML.getChildByTagName(mr_query_response, "response");
	    
	    page.appendChild(doc_.importNode(result_response, true));
	    return transformer_.transform(stylesheet, page);
	}

	// we have a doc list, so get the metadata - for now, get title.
	// can we dynamically decide what metadata to get?
	// get the Title metadata
	Element mr_metadata_message = doc_.createElement("message");
	Element mr_metadata_request = doc_.createElement("request");
	mr_metadata_message.appendChild(mr_metadata_request);

	mr_metadata_request.setAttribute("type", "query");
	to = GSPath.appendLink(collection, "MetadataRetrieve");
	mr_metadata_request.setAttribute("to", to);
	
	Element meta_content = doc_.createElement("content");
	mr_metadata_request.appendChild(meta_content);

	// the first part of the content is the doc list
	meta_content.appendChild(doc_.importNode(resource_list, true));

	// the second part of the content is the metadata list
	Element metadata_list = doc_.createElement("metadataList");
	Element title = doc_.createElement("element");
	title.setAttribute("name", "Title");
	metadata_list.appendChild(title);
	meta_content.appendChild(metadata_list);

	Element mr_metadata_response = (Element)mr_.process(mr_metadata_message);
	
	Element result_response = (Element)GSXML.getChildByTagName(mr_metadata_response, "response");
	
	page.appendChild(doc_.importNode(result_response, true));
	
	return transformer_.transform(stylesheet, page);
	
    }

    protected String fieldQuery(Element page, Element request) {
	return GSHTML.errorPage("field query not implemented");


    }
    protected String unknownQuery(Element page, Element request) {

	return GSHTML.errorPage("unknown query subtype - unknown query not implemented");
    }
}
