package org.greenstone.gsdl3.util;

import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

// other java classes
import java.io.File;
import java.util.HashMap;

public class Translate {

    protected ConfigVars config_=null;
    protected HashMap lang_map_=null;
    protected Document doc_=null;
    protected XMLConverter converter_ = null;
    protected String default_lang_=null;

    public Translate() {
	lang_map_ = new HashMap();
	converter_ = new XMLConverter();
	doc_ = converter_.newDOM();
    
    }

    public void setConfigVars(ConfigVars config) {
	config_ = config;
    }

    public void setDefaultLanguage(String lang) {
	default_lang_=lang;
	loadLanguage(lang);
    }
    /** load in a translation file to memory
     * later - extend this to reload even if present?
     */
    public boolean loadLanguage(String lang) {
	if (lang_map_.containsKey(lang)) {
	    return true;
	}

	File lang_file = GSFile.getLangFile(config_, lang);
	if (!lang_file.exists()) {
	    System.out.println("file "+lang_file.getPath()+" doesn't exist");
	    return false;
	}
	Document new_lang = converter_.getDOM(lang_file);
	lang_map_.put(lang, new_lang.getDocumentElement());
	return true;
    }

    public Element getLanguageTree(String lang) {
	return getLanguageTree(lang, default_lang_);
    }
    /** returns a new <translate> element - the nodes it contains are
     * shared (I think) */
    public Element getLanguageTree(String current_lang, 
				   String default_lang) {

	Element tree = doc_.createElement("translate");
	// load the current lang
	Element current_node = doc_.createElement("current");
	tree.appendChild(current_node);
	loadLanguage(current_lang);
	current_node.appendChild(doc_.importNode((Element)lang_map_.get(current_lang), true));

	if (default_lang==null) {
	    default_lang = default_lang_;
	}
	if (default_lang==null || default_lang.equals(current_lang)) {
	    // dont need to add the default tree
	    return tree;
	}

	// add the default stuff on
	Element default_node = doc_.createElement("default");
	tree.appendChild(default_node);
	loadLanguage(default_lang);
	default_node.appendChild(doc_.importNode((Element)lang_map_.get(default_lang), true));

	return tree;
    }
}
